@@ -5,6 +5,7 @@
 from typing import Optional, List, Union
 
 import requests
+from requests.auth import HTTPBasicAuth
 from requests.models import Response
 from requests_oauthlib.oauth2_session import OAuth2Session
 
@@ -26,6 +27,8 @@ from pyyoutube.models import (
     SubscriptionListResponse,
     I18nRegionListResponse,
     I18nLanguageListResponse,
+    MemberListResponse,
+    MembershipsLevelListResponse,
     VideoAbuseReportReasonListResponse,
 )
 from pyyoutube.utils.params_checker import enf_comma_separated, enf_parts
@@ -46,7 +49,7 @@ class Api(object):
 
         Now this api provide methods as follows:
             >>> api.get_authorization_url()
-            >>> api.exchange_code_to_access_token()
+            >>> api.generate_access_token()
             >>> api.refresh_token()
             >>> api.get_channel_info()
             >>> api.get_playlist_by_id()
@@ -81,7 +84,7 @@ class Api(object):
 
     BASE_URL = "https://www.googleapis.com/youtube/v3/"
     AUTHORIZATION_URL = "https://accounts.google.com/o/oauth2/v2/auth"
-    EXCHANGE_ACCESS_TOKEN_URL = "https://www.googleapis.com/oauth2/v4/token"
+    EXCHANGE_ACCESS_TOKEN_URL = "https://oauth2.googleapis.com/token"
     USER_INFO_URL = "https://www.googleapis.com/oauth2/v1/userinfo"
 
     DEFAULT_REDIRECT_URI = "https://localhost/"
@@ -101,10 +104,8 @@ class Api(object):
         client_secret: Optional[str] = None,
         api_key: Optional[str] = None,
         access_token: Optional[str] = None,
-        oauth_redirect_uri: Optional[str] = None,
         timeout: Optional[int] = None,
         proxies: Optional[dict] = None,
-        quota: Optional[int] = None,
     ) -> None:
         """
         This Api provide two method to work. Use api key or use access token.
@@ -119,17 +120,12 @@ class Api(object):
             access_token(str, optional):
                 If you not provide api key, you can do authorization to get an access token.
                 If all api key and access token provided. Use access token first.
-            oauth_redirect_uri(str, optional)
-                Determines how Google's authorization server sends a response to your app.
-                If not provide will use default https://localhost/
             timeout(int, optional):
                 The request timeout.
             proxies(dict, optional):
                 If you want use proxy, need point this param.
                 param style like requests lib style.
                 Refer https://2.python-requests.org//en/latest/user/advanced/#proxies
-            quota(int, optional):
-                if your key has more quota. you can point this. Default is 10000
 
         Returns:
             YouTube Api instance.
@@ -139,16 +135,11 @@ class Api(object):
         self._api_key = api_key
         self._access_token = access_token
         self._refresh_token = None  # This keep current user's refresh token.
-        self.oauth_redirect_uri = oauth_redirect_uri or self.DEFAULT_REDIRECT_URI
         self._timeout = timeout
         self.session = requests.Session()
-        self.scope = None
+        self._oauth_session = None
         self.proxies = proxies
 
-        self.quota = quota
-        if self.quota is None:
-            self.quota = self.DEFAULT_QUOTA
-
         if not (
             (self._client_id and self._client_secret)
             or self._api_key
@@ -165,12 +156,18 @@ class Api(object):
             self._timeout = self.DEFAULT_TIMEOUT
 
     def get_authorization_url(
-        self, scope: Optional[List[str]] = None, **kwargs
+        self,
+        redirect_uri: Optional[str] = None,
+        scope: Optional[List[str]] = None,
+        **kwargs,
     ) -> (str, str):
         """
         Build authorization url to do authorize.
 
         Args:
+            redirect_uri(str, optional)
+                Determines how Google's authorization server sends a response to your app.
+                If not provide will use default https://localhost/
             scope (list, optional)
                 The scope you want give permission.
                 If you not provide, will use default scope.
@@ -181,17 +178,19 @@ class Api(object):
             The uri you can open on browser to do authorize.
         """
 
-        self.scope = scope
-        if self.scope is None:
-            self.scope = self.DEFAULT_SCOPE
+        if redirect_uri is None:
+            redirect_uri = self.DEFAULT_REDIRECT_URI
 
-        session = OAuth2Session(
+        if scope is None:
+            scope = self.DEFAULT_SCOPE
+
+        self._oauth_session = OAuth2Session(
             client_id=self._client_id,
-            scope=self.scope,
-            redirect_uri=self.oauth_redirect_uri,
+            scope=scope,
+            redirect_uri=redirect_uri,
             state=self.DEFAULT_STATE,
         )
-        authorization_url, state = session.authorization_url(
+        authorization_url, state = self._oauth_session.authorization_url(
             self.AUTHORIZATION_URL,
             access_type="offline",
             prompt="select_account",
@@ -200,7 +199,7 @@ class Api(object):
 
         return authorization_url, state
 
-    def exchange_code_to_access_token(
+    def generate_access_token(
         self, authorization_response: str, return_json: bool = False
     ) -> Union[dict, AccessToken]:
         """
@@ -214,21 +213,24 @@ class Api(object):
                 False will return pyyoutube.AccessToken
 
         Return:
-            Retrieved access token's info,  pyyoutube.AccessToken instance.
+            Retrieved access token's info, pyyoutube.AccessToken instance.
         """
+        if not self._oauth_session:
+            raise PyYouTubeException(
+                ErrorMessage(
+                    status_code=ErrorCode.AUTHORIZE_URL_FIRST,
+                    message="You must get authorization url first",
+                )
+            )
 
-        session = OAuth2Session(
-            client_id=self._client_id,
-            redirect_uri=self.oauth_redirect_uri,
-            state=self.DEFAULT_STATE,
-        )
-        token = session.fetch_token(
+        token = self._oauth_session.fetch_token(
             self.EXCHANGE_ACCESS_TOKEN_URL,
             client_secret=self._client_secret,
             authorization_response=authorization_response,
+            proxies=self.proxies,
         )
-        self._access_token = session.access_token
-        self._refresh_token = session.token["refresh_token"]
+        self._access_token = self._oauth_session.access_token
+        self._refresh_token = self._oauth_session.token["refresh_token"]
         if return_json:
             return token
         else:
@@ -258,15 +260,17 @@ class Api(object):
                 )
             )
 
-        session = OAuth2Session(client_id=self._client_id)
-        auth = {
-            "client_id": self._client_id,
-            "client_secret": self._client_secret,
-        }
-        new_token = session.refresh_token(
-            self.EXCHANGE_ACCESS_TOKEN_URL, refresh_token=refresh_token, **auth
+        oauth_session = self._oauth_session
+        if oauth_session is None:
+            oauth_session = OAuth2Session(client_id=self._client_id)
+
+        auth = HTTPBasicAuth(self._client_id, self._client_secret)
+        new_token = oauth_session.refresh_token(
+            self.EXCHANGE_ACCESS_TOKEN_URL,
+            refresh_token=refresh_token,
+            auth=auth,
         )
-        self._access_token = session.access_token
+        self._access_token = oauth_session.access_token
         if return_json:
             return new_token
         else:
@@ -1783,7 +1787,6 @@ class Api(object):
         *,
         section_id: Union[str, list, tuple, set],
         parts: Optional[Union[str, list, tuple, set]] = None,
-        hl: Optional[str] = "en_US",
         return_json: Optional[bool] = False,
     ) -> Union[ChannelSectionResponse, dict]:
         """
@@ -1797,9 +1800,6 @@ class Api(object):
                 The resource parts for channel section you want to retrieve.
                 If not provide, use default public parts.
                 You can pass this with single part str, comma-separated parts str or a list,tuple,set of parts.
-            hl:
-                If provide this. Will return playlist's language localized info.
-                This value need https://developers.google.com/youtube/v3/docs/i18nLanguages.
             return_json:
                 The return data type. If you set True JSON data will be returned.
                 False will return a pyyoutube.ChannelSectionResponse instance.
@@ -1809,7 +1809,6 @@ class Api(object):
 
         args = {
             "id": enf_comma_separated(field="section_id", value=section_id),
-            "hl": hl,
             "part": enf_parts(resource="channelSections", value=parts),
         }
 
@@ -1827,7 +1826,6 @@ class Api(object):
         channel_id: Optional[str] = None,
         mine: bool = False,
         parts: Optional[Union[str, list, tuple, set]] = None,
-        hl: Optional[str] = "en_US",
         return_json: Optional[bool] = False,
     ) -> Union[ChannelSectionResponse, dict]:
         """
@@ -1843,9 +1841,6 @@ class Api(object):
                 The resource parts for channel section you want to retrieve.
                 If not provide, use default public parts.
                 You can pass this with single part str, comma-separated parts str or a list,tuple,set of parts.
-            hl:
-                If provide this. Will return playlist's language localized info.
-                This value need https://developers.google.com/youtube/v3/docs/i18nLanguages.
             return_json:
                 The return data type. If you set True JSON data will be returned.
                 False will return a pyyoutube.ChannelSectionResponse instance.
@@ -1854,7 +1849,6 @@ class Api(object):
         """
 
         args = {
-            "hl": hl,
             "part": enf_parts(resource="channelSections", value=parts),
         }
 
@@ -1942,6 +1936,133 @@ class Api(object):
         else:
             return I18nLanguageListResponse.from_dict(data)
 
+    def get_members(
+        self,
+        *,
+        parts: Optional[Union[str, list, tuple, set]] = None,
+        mode: Optional[str] = "all_current",
+        count: Optional[int] = 5,
+        limit: Optional[int] = 5,
+        page_token: Optional[str] = None,
+        has_access_to_level: Optional[str] = None,
+        filter_by_member_channel_id: Optional[Union[str, list, tuple, set]] = None,
+        return_json: Optional[bool] = False,
+    ) -> Union[MemberListResponse, dict]:
+        """
+        Retrieve a list of members for a channel.
+
+        Args:
+            parts ((str,list,tuple,set) optional):
+                The resource parts for member you want to retrieve.
+                If not provide, use default public parts.
+                You can pass this with single part str, comma-separated parts str or a list,tuple,set of parts.
+            mode:
+                The mode parameter indicates which members will be included in the API response.
+                Set the parameter value to one of the following values:
+                    - all_current (default): List current members, from newest to oldest. When this value is used,
+                        the end of the list is reached when the API response does not contain a nextPageToken.
+                    - updates : List only members that joined or upgraded since the previous API call.
+                        Note that the first call starts a new stream of updates but does not actually return any members.
+                        To start retrieving the membership updates, you need to poll the endpoint using the
+                        nextPageToken at your desired frequency.
+                        Note that when this value is used, the API response always contains a nextPageToken.
+            count (int, optional):
+                The count will retrieve videos data.
+                Default is 5.
+            limit (int, optional):
+                The maximum number of items each request retrieve.
+                For members, this should not be more than 1000.
+                Default is 5.
+            page_token (str, optional):
+                The token of the page of search result to retrieve.
+                You can use this retrieve point result page directly.
+                And you should know about the the result set for YouTube.
+            has_access_to_level (str, optional):
+                The hasAccessToLevel parameter value is a level ID that specifies the minimum level
+                that members in the result set should have.
+            filter_by_member_channel_id ((str,list,tuple,set) optional):
+                A list of channel IDs that can be used to check the membership status of specific users.
+                A maximum of 100 channels can be specified per call.
+            return_json (bool, optional):
+                The return data type. If you set True JSON data will be returned.
+                False will return a pyyoutube.MemberListResponse instance.
+        Returns:
+            MemberListResponse or original data
+        """
+
+        if count is None:
+            limit = 1000
+        else:
+            limit = min(count, limit)
+
+        args = {
+            "part": enf_parts(resource="members", value=parts),
+            "maxResults": limit,
+        }
+
+        if mode:
+            args["mode"] = mode
+
+        if page_token is not None:
+            args["pageToken"] = page_token
+
+        if has_access_to_level:
+            args["hasAccessToLevel"] = has_access_to_level
+
+        if filter_by_member_channel_id:
+            args["filterByMemberChannelId"] = enf_parts(
+                resource="filterByMemberChannelId",
+                value=filter_by_member_channel_id,
+                check=False,
+            )
+
+        res_data = self.paged_by_page_token(
+            resource="members",
+            args=args,
+            count=count,
+        )
+        if return_json:
+            return res_data
+        else:
+            return MemberListResponse.from_dict(res_data)
+
+    def get_membership_levels(
+        self,
+        *,
+        parts: Optional[Union[str, list, tuple, set]] = None,
+        return_json: Optional[bool] = False,
+    ) -> Union[MembershipsLevelListResponse, dict]:
+        """
+        Retrieve membership levels for a channel
+
+        Notes:
+            This requires your authorization.
+
+        Args:
+            parts ((str,list,tuple,set) optional):
+                The resource parts for membership level you want to retrieve.
+                If not provide, use default public parts.
+                You can pass this with single part str, comma-separated parts str or a list,tuple,set of parts.
+            return_json (bool, optional):
+                The return data type. If you set True JSON data will be returned.
+                False will return a pyyoutube.MembershipsLevelListResponse instance.
+
+        Returns:
+            MembershipsLevelListResponse or original data
+        """
+
+        args = {
+            "part": enf_parts(resource="membershipsLevels", value=parts),
+        }
+
+        resp = self._request(resource="membershipsLevels", args=args)
+        data = self._parse_response(resp)
+
+        if return_json:
+            return data
+        else:
+            return MembershipsLevelListResponse.from_dict(data)
+
     def get_video_abuse_report_reason(
         self,
         *,
